package com.jiankangyouyi.cloud.data.center.model.bean;

import com.jiankangyouyi.cloud.core.external.qiniu.QiniuClient;
import com.jiankangyouyi.cloud.data.center.api.model.entity.component.ExtensionUnit;
import com.jiankangyouyi.cloud.data.center.enums.QiniuBucketEnum;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.collections.CollectionUtils;
import org.bson.types.ObjectId;

import javax.persistence.Transient;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

/**
 * @author jingpeng
 * @description 搜索出的系统食物对象
 * @date 2020/11/10
 */

@Data
@NoArgsConstructor
public class FoodInfoBean implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 食物id
     */
    @Transient
    private ObjectId id;
    private String foodId;

    /**
     * 食物类型，系统食物，自定义食物
     */
    private String foodType;

    /**
     * 食物名称
     */
    private String foodName;

    /**
     * 图片地址
     */
    private String imageKey;
    private String imageUrl;

    /**
     * 单位
     */
    private String unit;

    /**
     * 扩展单位
     */
    private List<ExtensionUnit> extensionUnits;

    /**
     * 食谱对象，食物为半成品、成品时填充
     */
    private DietaryBean dietary;

    /**
     * 食谱原料质量比值
     */
    private FoodMassRatioBean massRatio;
    /**
     * 原始质量 最小值
     */
    protected Integer minQuantity;
    @Transient
    private int realMass;

    /**
     * 热量(kcal),按每100克计算
     */
    private Integer minQuantityEnergy;
    @Transient
    private int heat;

    /**
     * 脂肪含量
     */
    private double fat;
    /**
     * 碳水化合物含量
     */
    private double carbohydrate;
    /**
     * 蛋白质含量
     */
    private double protein;
    /**
     * 食物基础营养成分数据
     */
    private List<FoodPropertyInfo> foodPropertyInfos;

    /**
     * 是否显示承重按钮，度量衡
     */
    private String isMetrology;

    /**
     * 承重跳转标识 ，1 类别， 2明细
     */
    private String metrologyTag;


    /**
     * 根据食物数量 重新计算食物信息，按比例计算热量，食材，营养素等等
     */
    public void refreshFoodQuantity(int foodCount) {
        this.minQuantity = foodCount;
        this.minQuantityEnergy = calculation(foodCount, this.minQuantityEnergy).intValue();
        this.fat = calculation(foodCount, this.fat).doubleValue();
        this.carbohydrate = calculation(foodCount, this.carbohydrate).doubleValue();
        this.protein = calculation(foodCount, this.protein).doubleValue();
        //计算食材，主菜 配菜
        if (CollectionUtils.isNotEmpty(this.getDietary().getMajorMaterial())) {
            this.getDietary().getMajorMaterial().forEach(foodMaterialBean -> {
                foodMaterialBean.calculateCount(foodCount, this.unit);
            });
        }
        if (CollectionUtils.isNotEmpty(this.getDietary().getMinorMaterial())) {
            this.getDietary().getMinorMaterial().forEach(materialBean -> {
                materialBean.calculateCount(foodCount, this.unit);
            });
        }
        //食物基础成分
        if (CollectionUtils.isNotEmpty(this.getFoodPropertyInfos())) {
            this.getFoodPropertyInfos().forEach(propertyInfo -> {
                if (!"%".equals(propertyInfo.getPropertyUnit())) {
                    propertyInfo.setCount(calculation(foodCount, propertyInfo.getCount()).intValue());
                }
            });
        }
    }

    private BigDecimal calculation(int foodCount, double element) {

        return new BigDecimal(foodCount).multiply(BigDecimal.valueOf(element)).divide(new BigDecimal(100), 2,
                BigDecimal.ROUND_HALF_UP);

    }

    public void setRealMass(int realMass) {
        this.realMass = realMass;
        setMinQuantity(realMass);
    }

    public void setHeat(int heat) {
        this.minQuantityEnergy = heat;
        this.heat = heat;
    }

    public void setId(ObjectId id) {
        this.id = id;
        this.foodId = id.toHexString();
    }

    public void setImageKey(String imageKey) {
        QiniuClient qiniuClient = QiniuClient.getClient(QiniuBucketEnum.JY.getBucket());
        this.imageKey = imageKey;
        this.imageUrl = qiniuClient.imageKeyToImageUrl(imageKey);
    }

    /**
     * 食谱对象
     */
    @Data
    public static class DietaryBean {

        /**
         * 水作为
         **/
        private String waterType;

        /**
         * 工艺
         **/
        private String craft;

        /**
         * 主料
         */
        private List<FoodMaterialBean> majorMaterial;

        /**
         * 辅料
         */
        private List<FoodMaterialBean> minorMaterial;

        /**
         * 烹饪时间 单位 分钟
         */
        private int cookingTime;

        /**
         * 烹饪步骤
         */
        private List<FoodCookingStep> cookingSteps;

    }

    /**
     * 食材
     */
    @Data
    public static class FoodMaterialBean {
        /**
         * 食材ID
         */
        private String materialId;
        /**
         * 数量
         */
        private String count;
        /**
         * 食材名称
         */
        private String materialName;

        /**
         * 食物图片Key
         */
        private String imageUrl;

        /**
         * 类型
         */
        private String type;

        /**
         * 单位
         */
        private String unit;

        /**
         * 重新设定数量
         */
        public void calculateCount(int minQuantity, String unit) {
            if (this.count.isEmpty() || Double.parseDouble(this.count) == 0) {
                this.count = "适量";
                this.unit = "";
            } else {
                this.count = (new BigDecimal(count).divide(BigDecimal.valueOf(100), 2, BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(minQuantity))
                        .setScale(1, RoundingMode.DOWN).toBigInteger().toString());
                this.unit = unit;
            }
        }
    }

    /**
     * 烹饪步骤
     */
    @Data
    public static class FoodCookingStep {

        /**
         * 序号
         */
        private String stepSeq;

        /**
         * 图片
         */
        private String imageKey;

        /***
         * 图片
         */
        private String imageUrl;
        /**
         * 标题
         */
        private String title;

        /**
         * 内容
         */
        private String content;
    }

    /**
     * 食物质量比例
     */
    @Data
    public static class FoodMassRatioBean {
        /**
         * 谷类
         */
        private double guPercent;

        /**
         * 杂豆类
         */
        private double zdPercent;

        /**
         * 薯类
         */
        private double shuPercent;

        /**
         * 淀粉类
         */
        private double dfPercent;

        /**
         * 肉类
         */
        private double rouPercent;

        /**
         * 蔬菜类
         */
        private double scPercent;

        /**
         * 菌藻类
         */
        private double jzPercent;

        /**
         * 水果类
         */
        private double sgPercent;

        /**
         * 蛋类
         */
        private double danPercent;

        /**
         * 乳类
         */
        private double ruPercent;

        /**
         * 大豆类
         */
        private double ddPercent;

        /**
         * 坚果类
         */
        private double jgPercent;

        /**
         * 零食类
         */
        private double lsPercent;

        /**
         * 饮品类
         */
        private double ypPercent;

        /**
         * 调味品类
         */
        private double twpPercent;

        /**
         * 油脂类
         */
        private double yzPercent;
    }

    /**
     * 食物基础营养成分数据
     *
     * @author Ashon
     */
    @Builder
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    public static class FoodPropertyInfo {

        /**
         * 英文名称
         */
        private String propertyName;
        /**
         * 中文名称
         */
        private String chineseName;
        /**
         * 英文单位
         */
        private String propertyUnit;
        /**
         * 中文单位
         */
        private String chineseUnit;
        /**
         * 类型
         */
        private String type;
        /**
         * 数量
         */
        private double count;

    }

}
