package com.jiankangyouyi.cloud.food_match.model.dto.request.match;

import com.jiankangyouyi.cloud.core.validator.annotation.*;
import com.jiankangyouyi.cloud.data.center.api.enums.LactationStatusEnum;
import com.jiankangyouyi.cloud.data.center.api.model.entity.component.HealthProblemInfo;
import com.jiankangyouyi.cloud.data.center.api.model.entity.component.NutrientStateInfo;
import com.jiankangyouyi.cloud.food_match.api.enums.ExerciseLevelEnum;
import com.jiankangyouyi.cloud.food_match.enums.GenderRealEnum;
import com.jiankangyouyi.cloud.food_match.model.dto.request.AbstractAuthHttpApiRequestDTO;
import com.jiankangyouyi.cloud.food_match.model.dto.request.match.test.MatchTestParam;
import lombok.*;

import javax.validation.constraints.Pattern;
import java.util.List;
import java.util.Objects;

/**
 * @description: 配餐请求dto
 * @author: zyj
 * @email: zhaoyujie@jiankangyouyi.com
 * @date: 2021/7/26 3:32 下午
 */
@EqualsAndHashCode(callSuper = true)
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class APIMatchFoodV2RequestDTO extends AbstractAuthHttpApiRequestDTO {

    private static final long serialVersionUID = 1L;

    /**
     * 用户id
     */
    @NotEmptyHC
    @SizeHC(value = 64)
    private String userId;
    /**
     * 体重 kg
     */
    @NotEmptyHC
    @BigDecimalValHC
    @MinHC(25)
    @MaxHC(200)
    private String weight;
    /**
     * 身高 cm
     * @length 3
     */
    @NotEmptyHC
    @IntValHC
    @MinHC(100)
    @MaxHC(230)
    private String height;
    /**
     * 出生年月日 yyyy-MM-dd
     */
    @NotEmptyHC
    @DateValHC(pattern = "yyyy-MM-dd")
    private String birthday;
    /**
     * 性别 1:男2:女
     * @length 1
     */
    @NotEmptyHC
    @EnumHC(enumTypes = GenderRealEnum.class)
    private String gender;
    /**
     * 体力活动等级
     * @length 1
     */
    @NotEmptyHC
    @Pattern(regexp = "^[1-5]$", message = "activityLevel 不合法")
    private String activityLevel;
    /**
     * 运动等级
     * @length 1
     */
    @EnumHC(enumTypes = ExerciseLevelEnum.class)
    private String exerciseRiskLevel;

    /**
     * 疾病情况
     */
    private List<HealthProblemInfo> healthProblemInfos;

    /**
     * NULL 营养素状态信息 当健康问题包含 NUTRIENT_DEFICIENCY 时此处不可为空
     */
    private List<NutrientStateInfo> nutrientStateInfos;

    /**
     * NUll 分娩日期,性别为女性时才能赋值 yyyy-MM-dd
     */
    @DateValHC(pattern = "yyyy-MM-dd")
    private String deliveryDate;

    /**
     * NULL 哺乳状态，性别为女性时才能赋值
     * <p>
     * {@link com.jiankangyouyi.cloud.data.center.api.enums.LactationStatusEnum}
     */
    @EnumHC(enumTypes = LactationStatusEnum.class)
    @SizeHC(1)
    private String lactationStatus;

    /**
     * NULL 民族 （膳食使用时数据来源——字典——字典大类：食物库，字典子类：禁忌民族）
     */
    @SizeHC(value = 16)
    private String nation;
    /**
     * 所在的地域
     * （膳食使用时数据来源——字典——字典大类：食物库，字典子类：国家及地区）
     */
    @SizeHC(value = 16)
    private String area;

    /**
     * 喜好口味
     * （膳食使用时数据来源——字典——字典大类：食物库，字典子类：口味）
     */
    private List<String> taste;

    /**
     * 过敏原禁忌标签 {@link com.jiankangyouyi.cloud.data.center.api.enums.AllergenTabooTagEnum}
     */
    private List<String> allergenTabooTags;
    /**
     * 配餐日期
     */
    @NotEmptyHC
    @DateValHC(pattern = "yyyy-MM-dd")
    private String planningDate;

    /**
     * 是否强行重新配餐
     * @length 5
     */
    @BooleanValHC
    private String reMatch;


    /**
     * 糖尿病用药情况,不传默认4
     * {@link com.jiankangyouyi.cloud.data.center.api.enums.Type2DiabetesPharmacyEnum}
     */
    @SizeHC(1)
    private String type2DiabetesPharmacyCode;

    private Integer type;

    private Integer recommendedIntakeEnergy;


    /**
     * 测试
     */
    private MatchTestParam test;

    public MatchTestParam getTest() {
        if (Objects.isNull(test)) {
            test = new MatchTestParam();
        }
        return test;
    }
}
