package com.june.entitytodbdoc.utils;

import com.june.entitytodbdoc.dto.DocumentInfoDTO;
import org.apache.commons.collections4.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * Markdown Utils
 *
 * @author june
 */
public class MarkdownUtils {


    /**
     * 获取一个文档的数据库文档的markdown内容
     *
     * @param document 文档信息
     * @return markdown内容
     */
    public static List<String> getOneDocumentContent(DocumentInfoDTO document) {
        if (Objects.isNull(document)) {
            return Collections.emptyList();
        }
        // 返回的列表
        List<String> contentList = new LinkedList<>();
        // 获取 title
        contentList.add(getDocumentTitle(document, 2));
        List<DocumentInfoDTO.FieldInfoDTO> fieldList = document.getFieldList();
        if (CollectionUtils.isEmpty(fieldList)) {
            return contentList;
        }
        // 处理字段信息
        handleDocumentField(contentList,fieldList);
        return contentList;
    }

    /**
     * 处理文档字段信息
     *
     * @param contentList 文档内容
     * @param fieldList   字段列表
     */
    public static void handleDocumentField(List<String> contentList, List<DocumentInfoDTO.FieldInfoDTO> fieldList) {
        // 插入表头
        contentList.addAll(getTableHeader());
        // 处理一级字段
        fieldList.stream().map(MarkdownUtils::getOneFieldContent).forEach(contentList::add);
        // 处理子字段,先将所有子字段merge
        Map<List<String>, List<DocumentInfoDTO.FieldInfoDTO>> mergeChildrenFieldMap = new HashMap<>();
        mergeDocumentChildrenField(mergeChildrenFieldMap,fieldList);
        // 插入数据
        for (Map.Entry<List<String>, List<DocumentInfoDTO.FieldInfoDTO>> childrenEntry : mergeChildrenFieldMap.entrySet()) {
            // 插入标题
            contentList.addAll(childrenEntry.getKey());
            // 插入表头
            contentList.addAll(getTableHeader());
            // 插入字段内容
            childrenEntry.getValue().stream().map(MarkdownUtils::getOneFieldContent).forEach(contentList::add);
        }
    }

    /**
     * 获取一个字段的内容
     *
     * @param field 字段
     * @return {@link String}
     */
    public static String getOneFieldContent(DocumentInfoDTO.FieldInfoDTO field){
        return CollectionUtils.isEmpty(field.getChildrenFieldList())
                ? String.format("|%s|%s|%s|",field.getFieldName(),field.getFieldType(),field.getDescribe())
                : String.format("|%s|[%s](#%s)|[%s](#%s)|",field.getFieldName(),field.getFieldType(),field.getFiledId(),field.getDescribe(),field.getFiledId());
    }


    /**
     * 合并文档子字段信息
     *
     * @param fieldList 字段列表
     * @return {@link Map}<{@link String},{@link List}<{@link DocumentInfoDTO.FieldInfoDTO}>>
     */
    public static void mergeDocumentChildrenField(Map<List<String>, List<DocumentInfoDTO.FieldInfoDTO>> mergeChildrenFieldMap, List<DocumentInfoDTO.FieldInfoDTO> fieldList) {
        if (CollectionUtils.isEmpty(fieldList)) {
            return;
        }
        for (DocumentInfoDTO.FieldInfoDTO fieldInfoDTO : fieldList) {
            List<DocumentInfoDTO.FieldInfoDTO> childrenFieldList = fieldInfoDTO.getChildrenFieldList();
            if (CollectionUtils.isEmpty(childrenFieldList)) {
                continue;
            }
            // 添加子字段信息
            mergeChildrenFieldMap.put(getFieldTitle(fieldInfoDTO, 3), childrenFieldList);
            // 递归调用
            mergeDocumentChildrenField(mergeChildrenFieldMap, childrenFieldList);
        }
    }


    /**
     * 获取 表的 title
     *
     * @param document   文档信息对象
     * @param levelTitle 几级标题
     * @return title markdown
     */
    public static String getDocumentTitle(DocumentInfoDTO document, int levelTitle) {
        String documentDescribe = Optional.ofNullable(document.getDocumentDescribe()).orElse(document.getDocumentName());
        String titleLevelStr = getLevelTitleIndexStr(levelTitle);
        return String.format("%s %s (%s)", titleLevelStr, documentDescribe, document.getDocumentName());
    }

    /**
     * 获取 字段的 title
     *
     * @param field      字段信息对象
     * @param levelTitle 几级标题
     * @return title markdown
     */
    public static List<String> getFieldTitle(DocumentInfoDTO.FieldInfoDTO field, int levelTitle) {
        String fieldDescribe = Optional.ofNullable(field.getDescribe()).orElse(field.getFieldName());
        String titleLevelStr = getLevelTitleIndexStr(levelTitle);
        return Arrays.asList(
                getIndexDiv(field.getFiledId()),
                "",
                String.format("%s %s (%s)", titleLevelStr, fieldDescribe, field.getFieldName())
        );
    }


    /**
     * 获取 表格的 header
     *
     * @return table header markdown
     */
    public static List<String> getTableHeader() {
        return Arrays.asList(
                "",
                "|参数名|类型|说明|",
                "|----|----|----|"
        );
    }

    /**
     * 获取索引 div 内容
     *
     * @param id div Id
     * @return 内容字符串
     */
    private static String getIndexDiv(String id) {
        return String.format("<div id=\"%s\"/>", id);
    }

    /**
     * 获取标题级别 str
     *
     * @param levelTitle 级别
     * @return {@link String}
     */
    private static String getLevelTitleIndexStr(int levelTitle) {
        return IntStream.iterate(0, i -> i + 1).limit(levelTitle).mapToObj(i -> "#").collect(Collectors.joining());
    }


    public static void main(String[] args) {
        DocumentInfoDTO documentInfoDTO = new DocumentInfoDTO();
        documentInfoDTO.setDocumentName("v3.test");
        documentInfoDTO.setDocumentDescribe("测试文档");
        List<DocumentInfoDTO.FieldInfoDTO> fieldList = new LinkedList<>();


        List<DocumentInfoDTO.FieldInfoDTO> fieldList1 = new LinkedList<>();
        fieldList1.add(new DocumentInfoDTO.FieldInfoDTO("1-f1","String1","1-字段注释1"));
        fieldList1.add(new DocumentInfoDTO.FieldInfoDTO("1-f2","String1","1-字段注释2"));
        fieldList1.add(new DocumentInfoDTO.FieldInfoDTO("1-f3","String1","1-字段注释3"));
        List<DocumentInfoDTO.FieldInfoDTO> fieldList2 = new LinkedList<>();
        fieldList2.add(new DocumentInfoDTO.FieldInfoDTO("2-f1","String2","2-字段注释1",fieldList1));
        fieldList2.add(new DocumentInfoDTO.FieldInfoDTO("2-f2","String2","2-字段注释2"));
        fieldList2.add(new DocumentInfoDTO.FieldInfoDTO("2-f3","String2","2-字段注释3"));

        fieldList.add(new DocumentInfoDTO.FieldInfoDTO("f1","String","字段注释1",fieldList1));
        fieldList.add(new DocumentInfoDTO.FieldInfoDTO("f2","String","字段注释2",fieldList2));
        fieldList.add(new DocumentInfoDTO.FieldInfoDTO("f3","String","字段注释3"));
        documentInfoDTO.setFieldList(fieldList);
        List<String> onDocumentContent = getOneDocumentContent(documentInfoDTO);
        onDocumentContent.forEach(System.out::println);
    }

}
